local tokenget = 0 // Internal token counter.

local mobjscale = CV_RegisterVar({ 
	name = "kartgrow",				// Console variable to keep track
	defaultvalue = 0,				// for speed shoes replacement
	flags = CV_NETVAR,				// mechanic straight from SRB2Kart.
	PossibleValue = CV_YesNo
})

addHook("ThinkFrame", function()
	if gamestate == 4 then tokenget = 0 // Reset value if returned to title screen.
	end
	for player in players.iterate
		if (player.mo and player.mo.valid) and player.mo.skin == "windows" then
			token = 0 // No special stage. See "TouchSpecial" hook below.
			player.normalspeed = skins[player.mo.skin].normalspeed
			player.acceleration = skins[player.mo.skin].acceleration
			player.thrustfactor = skins[player.mo.skin].thrustfactor
			player.jumpfactor = skins[player.mo.skin].jumpfactor + (player.speed / 100)
			player.mo.destscale = FRACUNIT
			if player.speed > 20*FRACUNIT and player.powers[pw_super] == 0 then
				player.charflags = $ | SF_CANBUSTWALLS // Ram into the wall!
			else
				player.charflags = $ & ~SF_CANBUSTWALLS
			end
			if player.powers[pw_invulnerability] != 0 and player.powers[pw_sneakers] == 0 then
				// Kart mechanic.
				player.normalspeed = (skins[player.mo.skin].normalspeed * 4) / 3
				player.acceleration = (skins[player.mo.skin].acceleration * 4) / 3
			elseif player.powers[pw_sneakers] != 0 then
				// Before Kart's influence goes further...
				if mobjscale.value == 1 then
					player.mo.destscale = 2 * FRACUNIT // Grow!
					if player.powers[pw_sneakers] > 20 * TICRATE and player.powers[pw_super] == 0 then
						player.powers[pw_sneakers] = (player.powers[pw_sneakers] * 3) / 5
						if player.powers[pw_invulnerability] < player.powers[pw_sneakers] then
							player.powers[pw_invulnerability] = player.powers[pw_sneakers]
						end
					end
				else // Else, nothing exciting.
					player.thrustfactor = (skins[player.mo.skin].thrustfactor * 5) / 4
				end
			end
			if player.powers[pw_super] != 0 then
				player.charflags = $ | (SF_CANBUSTWALLS | SF_MULTIABILITY)
				if player.powers[pw_invulnerability] != 0 or player.powers[pw_sneakers] != 0 then
					player.powers[pw_invulnerability] = 0
					if mobjscale.value == 0 then
						player.powers[pw_sneakers] = 0
					end
				end
				if player.hyper == nil or player.hyper.capable == false then
					player.normalspeed = (skins[player.mo.skin].normalspeed * 3) / 2
					player.acceleration = (skins[player.mo.skin].acceleration * 4) / 3
					player.thrustfactor = (skins[player.mo.skin].thrustfactor * 61) / 51
				elseif player.hyper.capable == true then
					// Extreme overclocking by XMomentum! Careful!
					player.normalspeed = (skins[player.mo.skin].normalspeed * 311) / 100
					player.acceleration = (skins[player.mo.skin].acceleration * 3) / 2
					player.thrustfactor = (skins[player.mo.skin].thrustfactor * 3) / 2
				end
			else
				player.charflags = $ & ~SF_MULTIABILITY // No extra jumps if not Super.
			end
			if player.powers[pw_underwater] < 30*TICRATE or player.powers[pw_spacetime] < 30*TICRATE then
				player.powers[pw_underwater] = 30*TICRATE // We already have machines underwater.
				player.powers[pw_spacetime] = 30*TICRATE // And it's cold in space.
			end
			if player.mo.destscale > player.mo.scale then
				if not (S_SoundPlaying(player.mo, sfx_kc5a)) then
					S_StartSound(player.mo, sfx_kc5a)
				end
			elseif player.mo.destscale < player.mo.scale then
				if not (S_SoundPlaying(player.mo, sfx_kc59)) then
					S_StartSound(player.mo, sfx_kc59)
				end
			end
			if maptol == TOL_SP then // Fix emeroods if force-switched maps in Single Player.
				if emeralds == 0 and tokenget == 1 then 
					emeralds = 1
					CONS_Printf(player, "Correcting emerald count: 1")
				elseif emeralds == 0 and tokenget == 2 then
					emeralds = 3
					CONS_Printf(player, "Correcting emerald count: 2")
				elseif emeralds == 0 and tokenget == 3 then
					emeralds = 7
					CONS_Printf(player, "Correcting emerald count: 3")
				elseif emeralds == 0 and tokenget == 4 then
					emeralds = 15
					CONS_Printf(player, "Correcting emerald count: 4")
				elseif emeralds == 0 and tokenget == 5 then
					emeralds = 31
					CONS_Printf(player, "Correcting emerald count: 5")
				elseif emeralds == 0 and tokenget == 6 then
					emeralds = 63
					CONS_Printf(player, "Correcting emerald count: 6")
				elseif emeralds == 0 and tokenget == 7 then
					emeralds = 127
					CONS_Printf(player, "Correcting emerald count: 7")
				end
			end
			// Transform at any point as long as you have rings and emeralds,
			// regardless of a shield being present. Use wisely.
			if player.cmd.buttons == BT_CUSTOM3 and player.powers[pw_super] == 0 then
				if player.rings >= 50 and emeralds >= 127 then
					P_DoSuperTransformation(player, false)
				end
			end
		end
	end
end)

// Token-to-emerald compilation/conversion.
// Will only work with Windows (the character, not your OS).
// The character is not planned for use in NiGHTS special stages,
// so this is implemented as an alternative method of getting emeralds.
addHook("TouchSpecial", function()
	for player in players.iterate
		if (player.mo and player.mo.valid) and player.mo.skin == "windows" then
			if tokenget < 7 and emeralds < 127 then
				tokenget = tokenget + 1
				if emeralds == 0 or tokenget == 1 then
					emeralds = 1
					player.continues = $ + 1
					CONS_Printf(player, "Chaos Emeralds: 1")
				elseif emeralds == 1 or tokenget == 2 then
					emeralds = 3
					player.continues = $ + 1
					CONS_Printf(player, "Chaos Emeralds: 2")
				elseif emeralds == 3 or tokenget == 3 then
					emeralds = 7
					player.continues = $ + 1
					CONS_Printf(player, "Chaos Emeralds: 3")
				elseif emeralds == 7 or tokenget == 4 then
					emeralds = 15
					player.continues = $ + 1
					CONS_Printf(player, "Chaos Emeralds: 4")
				elseif emeralds == 15 or tokenget == 5 then
					emeralds = 31
					player.continues = $ + 1
					CONS_Printf(player, "Chaos Emeralds: 5")
				elseif emeralds == 31 or tokenget == 6 then
					emeralds = 63
					player.continues = $ + 1
					CONS_Printf(player, "Chaos Emeralds: 6")
				elseif emeralds == 63 or tokenget == 7 then
					emeralds = 127
					player.continues = $ + 1
					CONS_Printf(player, "You have them all!")
				end
			// Give rings from tokens instead if already have all emeralds.
			// So, like S3&K. Also set local token counter.
			elseif tokenget != 7 and emeralds >= 127
				tokenget = 7
				player.rings = player.rings + 50
			end
		end
	end
end, MT_TOKEN)

// Set up a value to check against for custom jingles.
local mplayer = CV_RegisterVar({
	name = "canyon.mid",
	defaultvalue = 1,
	flags = CV_SHOWMODIF,
	PossibleValue = CV_YesNo
})

// Script by Katmint.
// Edited w/ variable check.

addHook("MusicChange", function(oldname, newname)
	if consoleplayer
		local character = skins[consoleplayer.skin].name
		if character == "windows" then
			if newname == "_super" and mplayer.value != 0
				return "ERRORMIX"
			elseif newname == "_inv" and mplayer.value != 0
				return "WINV"
			elseif newname == "_shoes" 
				if mobjscale.value == 1 then return "KGROW"
				elseif mplayer.value != 0 return "WINFAST"
				end
			elseif newname == "HYPR" and mplayer.value != 0
				return "HYPWIN" // Hello, XMomentum.
			end
		end
	end
end)